/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.report;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import it.tidalwave.util.AsException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.observation.Location;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.observation.ObservationSet;
import it.tidalwave.mobile.location.LocationPreferences;
import it.tidalwave.semantic.document.Document;
import it.tidalwave.bluebill.mobile.preferences.GeneralPreferences;
import org.openide.util.NbBundle;
import static it.tidalwave.role.PlainTextRenderable.PlainTextRenderable;
import static it.tidalwave.bluebill.mobile.observation.ui.ObservationItemRenderingOption.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TextReportGenerator extends ReportGenerator
  {
    private static final Class<TextReportGenerator> _ = TextReportGenerator.class;
    
    private int previousDay = 0;

    private final Calendar calendar = Calendar.getInstance();

    private final List<String> strings = new ArrayList<String>();

    @Nonnull
    private final GeneralPreferences generalPreferences = Locator.find(GeneralPreferences.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TextReportGenerator() 
      {
        super("Text", Document.PLAIN_TEXT_MIME_TYPE);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void postVisit (final @Nonnull ObservationSet observationSet)
      {
        buffer.append(NbBundle.getMessage(_, "signature"));
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void preVisit (final @Nonnull Observation observation)
      {
        final Date date = observation.getDate();
        final Location location = observation.getLocation();
        
        calendar.setTime(date);
        final int thisDay = calendar.get(Calendar.DAY_OF_YEAR);

        if (previousDay != thisDay)
          {
            buffer.append(generalPreferences.formatDateAndTime(date));
          }
        else
          {
            buffer.append(generalPreferences.formatTime(date));
          }

        previousDay = thisDay;

        buffer.append(" ").append(location.getDisplayName());

        try
          {
            final LocationPreferences locationPreferences = Locator.find(LocationPreferences.class);
            buffer.append(" - ").append(locationPreferences.format(location.as(Range)));
          }
        catch (AsException e)
          {
            // Range not available, never mind
          }

        buffer.append("\n");
        strings.clear();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void visit (final @Nonnull ObservationItem observationItem)
      {
        strings.add("* " + observationItem.as(PlainTextRenderable).render(CARDINALITY, GENDER, MODE, NOTE_FULL));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void postVisit (final @Nonnull Observation observation)
      {
        Collections.sort(strings);

        for (final String string : strings)
          {
            buffer.append(string).append("\n");
          }

        buffer.append("\n");
        strings.clear();
      }
  }
