/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.text;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Locale;
import java.text.ChoiceFormat;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ObservationCountFormat
  {
    private static final Class<ObservationCountFormat> _ = ObservationCountFormat.class;
    
    protected int initializationCount = 0; // for testing

    @CheckForNull
    protected Locale locale;

    private ChoiceFormat choice;

    @Nonnull
    public String format (final int n)
      {
        initialize();
        return String.format(choice.format(n), n);
      }

    private synchronized void initialize()
      {
        if ((locale == null) || !locale.equals(Locale.getDefault()))
          {
            initializationCount++;
            locale = Locale.getDefault();

            final double[] limits = { 0, 1, 2 };
            final String[] parts = new String[]
              {
                getMessage(_, "noObservations"),
                getMessage(_, "oneObservation"),
                getMessage(_, "manyObservations")
              };

            choice = new ChoiceFormat(limits, parts);
          }
      }
  }
