/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.ui;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import it.tidalwave.mobile.util.ProgressListener;
import it.tidalwave.mobile.util.ProgressListenerSupport;
import it.tidalwave.mobile.ui.QuestionWithFeedback;
import it.tidalwave.bluebill.mobile.observation.report.CsvReportGenerator;
import it.tidalwave.bluebill.mobile.observation.report.KmlReportGenerator;
import it.tidalwave.bluebill.mobile.observation.report.ReportFactory;
import it.tidalwave.mobile.util.ExceptionReporter;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.ToString;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@NotThreadSafe @ToString
public abstract class ReportQuestionWithFeedback extends QuestionWithFeedback 
  {
    private final static Logger log = LoggerFactory.getLogger(ReportQuestionWithFeedback.class);
    
    private final static Class<ReportQuestionWithFeedback> _ = ReportQuestionWithFeedback.class;
    
    private static final ProgressListener DEFAULT_PROGRESS_LISTENER = new ProgressListenerSupport(); 
 
    @Nonnull
    protected ProgressListener progressListener = DEFAULT_PROGRESS_LISTENER; 
    
    @Getter(AccessLevel.PROTECTED)
    private ReportFactory reportFactory = new ReportFactory();
    
    // FIXME: discover them by Lookup and put in a list; then setXXXAttachment() takes an index and enable them by index
    private final KmlReportGenerator kmlReportGenerator = new KmlReportGenerator();
    
    private final CsvReportGenerator csvReportGenerator = new CsvReportGenerator();
    
    @Getter
    private final List<String> options = Collections.unmodifiableList(Arrays.asList
      (
        getMessage(_, "attachKML"),
        getMessage(_, "attachCSV")
      ));

    public ReportQuestionWithFeedback (final @Nonnull String title)
      {
        super(title, "");  
      }

    public void setKmlAttachment (final boolean kmlAttachment)
      {
        log.info("setKmlAttachment({})", kmlAttachment);
        
        if (kmlAttachment)
          {
            reportFactory = reportFactory.with(kmlReportGenerator);
          }
        else
          {
            reportFactory = reportFactory.without(kmlReportGenerator);
          }
      }

    public void setCsvAttachment (final boolean csvAttachment)
      {
        log.info("setCsvAttachment({})", csvAttachment);        
          
        if (csvAttachment)
          {
            reportFactory = reportFactory.with(csvReportGenerator);
          }
        else
          {
            reportFactory = reportFactory.without(csvReportGenerator);
          }
      }

    // FIXME: the ProgressListener has no meaning here, as we're a question feedback...
    public void setProgressListener (final @Nonnull ProgressListener progressListener) 
      {
        this.progressListener = progressListener;
      }

    @Override
    public void onConfirm()
      {
        final Thread thread = new Thread() // FIXME: perhaps it should be the subclass to decide?
          {
            @Override
            public void run()
              {
                try
                  {
                    onConfirmInBackground();
                  }
                catch (Throwable t)
                  {
                    ExceptionReporter.reportException(t);                         
                  }
              }
          };

        thread.start();
      }
    
    protected abstract void onConfirmInBackground()
      throws Exception;
  }